'use strict';

var _index = require('../../index');

var _index2 = _interopRequireDefault(_index);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _mocks = require('../../mocks');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('A database query', function () {

  var db = void 0;
  var executeQuery = void 0;

  beforeEach(function () {
    db = new _index2.default({
      engines: {
        sql: _mocks.queryEngine
      }
    });
    executeQuery = (0, _expect.spyOn)(_mocks.queryEngine, 'executeQuery');
  });
  afterEach(function () {
    executeQuery.restore();
  });

  it('should be a function', function () {
    (0, _expect2.default)(db.query).toBeA(Function);
  });

  it('should throw if it cannot find query engine', function () {
    var query = function query() {
      return db.query('string', {
        engine: 'engine'
      });
    };
    (0, _expect2.default)(query).toThrow(Error);
  });

  it('should call the correct query engine', function () {
    var query = 'query';
    var options = {
      engine: 'sql'
    };
    db.query(query, options);
    (0, _expect2.default)(executeQuery).toHaveBeenCalledWith(query, db);
  });

  it('should return correct value', function () {

    executeQuery.andReturn('cheese');

    var result = db.query(['users'], {
      engine: 'sql'
    });
    (0, _expect2.default)(result).toEqual('cheese');
  });
}); /* eslint-env mocha */