'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _graphCrdt = require('graph-crdt');

var _mocks = require('../../mocks');

var _context20 = require('../context');

var _context21 = _interopRequireDefault(_context20);

var _root = require('../root');

var _root2 = _interopRequireDefault(_root);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Database', function () {
  var settings = _root2.default.configuration;
  var db = void 0;

  beforeEach(function () {
    db = (0, _root2.default)();
  });

  it('is an object', function () {
    (0, _expect2.default)(db).toBeAn(Object);
  });

  it('generates a database configuration', function () {
    var config = db[settings];
    (0, _expect2.default)(config).toBeAn(Object);
  });

  it('creates a new graph', function () {
    (0, _expect2.default)(db).toBeA(_graphCrdt.Graph);
  });

  it('instantiates a new router', function () {
    var db = (0, _root2.default)({
      router: _mocks.createRouter
    });

    (0, _expect2.default)(db.router).toBeA(_mocks.Router);

    (0, _expect2.default)(_mocks.createRouter).toHaveBeenCalled();
    (0, _expect2.default)(_mocks.createRouter).toHaveBeenCalledWith(db, db[settings]);
  });

  describe('read', function () {
    var router = void 0;

    beforeEach(function () {
      router = (0, _mocks.createRouter)();

      db = (0, _root2.default)({
        router: (0, _expect.createSpy)().andReturn(router)
      });
    });

    it('returns null if nothing is found', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var settings;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return db.read('settings');

            case 2:
              settings = _context.sent;

              (0, _expect2.default)(settings).toBe(null);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('calls the router', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var _router$pull$calls$0$, read;

      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return db.read('lobby');

            case 2:

              (0, _expect2.default)(router.pull).toHaveBeenCalled();
              _router$pull$calls$0$ = (0, _slicedToArray3.default)(router.pull.calls[0].arguments, 1), read = _router$pull$calls$0$[0];


              (0, _expect2.default)(read).toBeAn(Object);
              (0, _expect2.default)(read.key).toBe('lobby');

            case 6:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('resolves with the router response', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var data, result;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              data = _graphCrdt.Node.from({ response: 'router' }).toJSON();

              router.pull.andReturn(data);

              _context3.next = 4;
              return db.read('weather');

            case 4:
              result = _context3.sent;

              (0, _expect2.default)(result).toBeA(_context21.default);
              (0, _expect2.default)(result.value('response')).toBe('router');

            case 7:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));
  });

  describe('commit', function () {
    var graph = void 0,
        node = void 0,
        beforeWrite = void 0,
        afterWrite = void 0,
        storage = void 0,
        router = void 0;
    var Identity = function Identity(value) {
      return value;
    };

    beforeEach(function () {
      beforeWrite = (0, _expect.createSpy)().andCall(Identity);
      afterWrite = (0, _expect.createSpy)().andCall(Identity);
      graph = new _graphCrdt.Graph();
      node = new _graphCrdt.Node();

      storage = new _mocks.Storage();
      storage.write = (0, _expect.createSpy)();

      router = new _mocks.Router();

      graph.merge((0, _defineProperty3.default)({}, node, node));

      db = (0, _root2.default)({
        hooks: {
          before: {
            write: beforeWrite
          },
          after: {
            write: afterWrite
          }
        },

        storage: [storage],

        router: (0, _expect.createSpy)().andReturn(router)
      });
    });

    it('returns a promise', function () {
      var result = db.commit(graph);

      (0, _expect2.default)(result).toBeA(_promise2.default);
    });

    it('triggers the before write pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var _beforeWrite$calls$0$, pipe;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(beforeWrite).toHaveBeenCalled();

              _beforeWrite$calls$0$ = (0, _slicedToArray3.default)(beforeWrite.calls[0].arguments, 1), pipe = _beforeWrite$calls$0$[0];

              (0, _expect2.default)(pipe.update).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(pipe.update))).toEqual([].concat((0, _toConsumableArray3.default)(graph)));

            case 6:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    it('calls storage drivers', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
      var _storage$write$calls$, write;

      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(storage.write).toHaveBeenCalled();
              _storage$write$calls$ = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$[0];


              (0, _expect2.default)(write.graph).toBeA(_graphCrdt.Graph);

            case 5:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    })));

    it('calls the router', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      var _router$push$calls$0$, config;

      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(router.push).toHaveBeenCalled();

              _router$push$calls$0$ = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), config = _router$push$calls$0$[0];


              (0, _expect2.default)(config).toBeAn(Object);
              (0, _expect2.default)(config.update).toBeA(_graphCrdt.Graph);

            case 6:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    })));

    it('passes the full state of each node to storage', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
      var update, node, _storage$write$calls$2, write, state;

      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return db.write('state', { existing: true });

            case 2:
              update = new _graphCrdt.Graph();
              node = new _graphCrdt.Node({ uid: 'state' });

              node.merge({ new: true });
              update.merge((0, _defineProperty3.default)({}, node, node));
              storage.write.reset();
              _context7.next = 9;
              return db.commit(update);

            case 9:

              (0, _expect2.default)(storage.write).toHaveBeenCalled();
              _storage$write$calls$2 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$2[0];
              state = write.graph.value('state');


              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(state))).toEqual([].concat((0, _toConsumableArray3.default)(_graphCrdt.Node.from({
                existing: true,
                new: true
              }))));

            case 13:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    })));

    it('only contains nodes which updated', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      var update, _storage$write$calls$3, write;

      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              update = new _graphCrdt.Graph();
              _context8.next = 3;
              return db.write('unrelated', { ignore: 'please' });

            case 3:
              storage.write.reset();
              _context8.next = 6;
              return db.commit(update);

            case 6:
              _storage$write$calls$3 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$3[0];

              (0, _expect2.default)(write.graph.value('unrelated')).toNotExist();

            case 8:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));

    it('uses the storage options if given', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return db.commit(graph, { storage: [] });

            case 2:

              (0, _expect2.default)(storage.write).toNotHaveBeenCalled();

            case 3:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));

    it('passes the deltas to plugins', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10() {
      var _router$push$calls$0$2, push, _storage$write$calls$4, write;

      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return db.commit(graph);

            case 2:
              _router$push$calls$0$2 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), push = _router$push$calls$0$2[0];

              (0, _expect2.default)(push.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(push.update).toBeA(_graphCrdt.Graph);

              _storage$write$calls$4 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$4[0];

              (0, _expect2.default)(write.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(write.update).toBeA(_graphCrdt.Graph);

            case 8:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, undefined);
    })));

    it('only sends the changes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11() {
      var weather, changes, _router$push$calls$0$3, push;

      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              weather = new _graphCrdt.Node({ uid: 'weather' });

              weather.merge({ condition: 'sunny', temp: 65 });
              db.merge((0, _defineProperty3.default)({}, weather, weather));

              changes = new _graphCrdt.Graph();

              changes.merge(db);
              changes.value(weather).merge({ temp: 70 });
              _context11.next = 8;
              return db.commit(changes);

            case 8:
              _router$push$calls$0$3 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), push = _router$push$calls$0$3[0];

              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(push.update)).length).toBe(1);
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(push.update.value('weather')))).toEqual([['temp', 70]]);

            case 11:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, undefined);
    })));

    it('caches the value after pipeline validation', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee12() {
      var node, cached;
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              node = _graphCrdt.Node.from({ updated: true });

              graph.merge((0, _defineProperty3.default)({}, node, node));
              _context12.next = 4;
              return db.commit(graph);

            case 4:
              cached = db.value(String(node));

              (0, _expect2.default)(cached).toExist();
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(cached))).toEqual([].concat((0, _toConsumableArray3.default)(node)));

            case 7:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, undefined);
    })));

    it('triggers the after write pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee13() {
      var _afterWrite$calls$0$a, write;

      return _regenerator2.default.wrap(function _callee13$(_context13) {
        while (1) {
          switch (_context13.prev = _context13.next) {
            case 0:
              _context13.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(afterWrite).toHaveBeenCalled();
              _afterWrite$calls$0$a = (0, _slicedToArray3.default)(afterWrite.calls[0].arguments, 1), write = _afterWrite$calls$0$a[0];


              (0, _expect2.default)(write.update).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(write.graph).toBeA(_graphCrdt.Graph);

            case 6:
            case 'end':
              return _context13.stop();
          }
        }
      }, _callee13, undefined);
    })));

    it('returns the deltas', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee14() {
      var result;
      return _regenerator2.default.wrap(function _callee14$(_context14) {
        while (1) {
          switch (_context14.prev = _context14.next) {
            case 0:
              node.merge({ commit: null });
              _context14.next = 3;
              return db.commit(graph);

            case 3:
              result = _context14.sent;


              (0, _expect2.default)(result).toBeAn(Object);
              (0, _expect2.default)(result.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(result.update).toBeA(_graphCrdt.Graph);

            case 7:
            case 'end':
              return _context14.stop();
          }
        }
      }, _callee14, undefined);
    })));

    it('transforms nodes into DB contexts', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee15() {
      var _node$meta, uid;

      return _regenerator2.default.wrap(function _callee15$(_context15) {
        while (1) {
          switch (_context15.prev = _context15.next) {
            case 0:
              node.merge({ count: 1 });
              _context15.next = 3;
              return db.commit(graph);

            case 3:
              _node$meta = node.meta(), uid = _node$meta.uid;

              (0, _expect2.default)(db.value(uid)).toBeA(_context21.default);

            case 5:
            case 'end':
              return _context15.stop();
          }
        }
      }, _callee15, undefined);
    })));
  });

  describe('write', function () {
    it('saves to the graph', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee16() {
      var node, name;
      return _regenerator2.default.wrap(function _callee16$(_context16) {
        while (1) {
          switch (_context16.prev = _context16.next) {
            case 0:
              _context16.next = 2;
              return db.write('user', { name: 'Jesse' });

            case 2:
              node = db.value('user');


              (0, _expect2.default)(node).toBeA(_context21.default);

              _context16.next = 6;
              return node.read('name');

            case 6:
              name = _context16.sent;

              (0, _expect2.default)(name).toBe('Jesse');

            case 8:
            case 'end':
              return _context16.stop();
          }
        }
      }, _callee16, undefined);
    })));

    it('does not overwrite, instead merges', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee17() {
      var user;
      return _regenerator2.default.wrap(function _callee17$(_context17) {
        while (1) {
          switch (_context17.prev = _context17.next) {
            case 0:
              _context17.next = 2;
              return db.write('user', { name: 'Bob' });

            case 2:
              _context17.next = 4;
              return db.write('user', { handle: 'BobTheGreat' });

            case 4:
              user = db.value('user');
              _context17.t0 = _expect2.default;
              _context17.next = 8;
              return user.read('name');

            case 8:
              _context17.t1 = _context17.sent;
              (0, _context17.t0)(_context17.t1).toBe('Bob');
              _context17.t2 = _expect2.default;
              _context17.next = 13;
              return user.read('handle');

            case 13:
              _context17.t3 = _context17.sent;
              (0, _context17.t2)(_context17.t3).toBe('BobTheGreat');

            case 15:
            case 'end':
              return _context17.stop();
          }
        }
      }, _callee17, undefined);
    })));

    it('updates node fields', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee18() {
      var user;
      return _regenerator2.default.wrap(function _callee18$(_context18) {
        while (1) {
          switch (_context18.prev = _context18.next) {
            case 0:
              _context18.next = 2;
              return db.write('user', { followers: 50 });

            case 2:
              user = _context18.sent;
              _context18.next = 5;
              return db.write('user', { followers: 49 });

            case 5:
              _context18.t0 = _expect2.default;
              _context18.next = 8;
              return user.read('followers');

            case 8:
              _context18.t1 = _context18.sent;
              (0, _context18.t0)(_context18.t1).toBe(49);
              _context18.next = 12;
              return db.write('user', { followers: 51 });

            case 12:
              _context18.t2 = _expect2.default;
              _context18.next = 15;
              return user.read('followers');

            case 15:
              _context18.t3 = _context18.sent;
              (0, _context18.t2)(_context18.t3).toBe(51);

            case 17:
            case 'end':
              return _context18.stop();
          }
        }
      }, _callee18, undefined);
    })));

    it('returns the written context', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee19() {
      var user, name;
      return _regenerator2.default.wrap(function _callee19$(_context19) {
        while (1) {
          switch (_context19.prev = _context19.next) {
            case 0:
              _context19.next = 2;
              return db.write('user', {
                name: 'Bob'
              });

            case 2:
              user = _context19.sent;


              (0, _expect2.default)(user).toBeA(_context21.default);
              (0, _expect2.default)(user).toBe(db.value('user'));

              _context19.next = 7;
              return user.read('name');

            case 7:
              name = _context19.sent;

              (0, _expect2.default)(name).toBe('Bob');

            case 9:
            case 'end':
              return _context19.stop();
          }
        }
      }, _callee19, undefined);
    })));
  });

  describe('API extensions', function () {
    var extend = function extend(api) {
      return (0, _root2.default)({
        extend: { root: api }
      });
    };

    it('are added to the root', function () {
      var db = extend({
        method: function method() {
          return true;
        },
        prop: 'yep'
      });

      (0, _expect2.default)(db.method).toBeA(Function);
      (0, _expect2.default)(db.prop).toBe('yep');
    });

    it('are non-enumerable', function () {
      var db = extend({ prop: 'value' });
      var keys = (0, _keys2.default)(db);
      (0, _expect2.default)(keys).toNotContain('prop');
    });

    it('are immutable', function () {
      var db = extend({ stuff: 'original' });
      (0, _expect2.default)(function () {
        db.stuff = 'something else';
      }).toThrow(Error);
    });
  });
}); /* eslint-env mocha */